/*
6
300 450
70 50 30
120 20 20
270 40 10
250 85 20
220 30 30
380 100 100

      LIGHT   , , 3  2001 .
   ------------------------------------------------------------------

   :       
   (Bx, By)  N (< 500)      ,
    i-         (Cix, Ciy)
       Ri.      ,  
          .
     LIGHT.EXE,      
   ,      .

   :    - N -   ,
           - Bx  By,
         N       - Cix, Ciy, Ri,
          Ciy + Ri < By.
         
              -  10 000.

   :    ,   ,
                .
                X-.


   : 1)        ,
                   
            .
            
            2)    . ,   
                -  0.01,   
                       .

   1)  . B (bx, by)    , . C (cx, cy) 
      N-   r - . .
   
         . B      . T (tx, ty).
        =  TBC,  = ( BC,  y = By ).
      
   :
      tx = bx - BT*cos( - ), ty = by - BT*sin( - ),
    CTB    
      tx = bx - BT*cos( + ), ty = by - BT*sin( + ),
    CTB   .

        .   :
      cos( - ) = cos()*cos() + sin()*sin();
      cos( + ) = cos()*cos() - sin()*sin();

      sin( - ) = sin()*cos() - cos()*sin();
      sin( + ) = sin()*cos() + cos()*sin();
   
      cos() = (Bx - Cx) / BC; cos() = BT / BC;
      sin() = (By - Cy) / BC; sin() = r / BC;

     -   :
      Tx = Bx - (BT / BC2)*( (Bx - Cx)*BT + (By - Cy)*r );
      Ty = By - (BT / BC2)*( (By - Cy)*BT - (Bx - Cx)*r );
       
      Tx = Bx - (BT / BC2)*( (Bx - Cx)*BT - (By - Cy)*r );
      Ty = By - (BT / BC2)*( (By - Cy)*BT + (Bx - Cx)*r );
    .

           - . B 
   . . .      BT   Ox.
         BT (y = a*x + b) y  0,
   :
      x = Bx - By*(Bx - Tx) / (By - Ty);


   2)     ,    .
   
   2.1)      X-.
   
   2.2)        (  -   ),
       X,   ,        
   ,     .
   
   2.3)         
    1        ,   
       .  ,      
     1,  ,      
           = 0 =>   .

    .


   :      , /  
     ()  LIGHT.IN / LIGHT.OUT,  LIGHT.C / .

                                                
                                               E-mail: mvalkanov@yahoo.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MAX_N 500

typedef struct { long int x, y; } CPnt;
typedef struct { CPnt c; long int r; } CCircle;
typedef struct { double x, y; } CPntDouble;
typedef struct { double b, e; } CInterval;
typedef struct { double x; char cKind; int nInterval; } CEnd;

int n;
CPnt b;

int nIntervalCnt;
CInterval vIntervals[MAX_N];

int nFinalIntervalCnt;
CInterval vFinalIntervals[MAX_N];

int nEndCnt;
CEnd vEnds[MAX_N*2];

void pAddCircle( CCircle * c )
{
   CPntDouble t1, t2;
   double k1, k2;
   long int BC2; double BT;

   BC2 = (b.x - c->c.x) * (b.x - c->c.x) + (b.y - c->c.y) * (b.y - c->c.y);
   BT = sqrt( BC2 - c->r * c->r );

   t1.x = b.x - (BT / BC2)*( (b.x - c->c.x)*BT + (b.y - c->c.y)*c->r );
   t1.y = b.y - (BT / BC2)*( (b.y - c->c.y)*BT - (b.x - c->c.x)*c->r );

   t2.x = b.x - (BT / BC2)*( (b.x - c->c.x)*BT - (b.y - c->c.y)*c->r );
   t2.y = b.y - (BT / BC2)*( (b.y - c->c.y)*BT + (b.x - c->c.x)*c->r );

   k1 = b.x - b.y*(b.x - t1.x) / (b.y - t1.y);
   k2 = b.x - b.y*(b.x - t2.x) / (b.y - t2.y);

   vIntervals[ nIntervalCnt ].b = k1; vIntervals[ nIntervalCnt++ ].e = k2;
}

int fIntervalCmp( const void * a, const void * b )
{
   CInterval * A = (CInterval *) a, * B = (CInterval *) b;

   return ( A->b < B->b ) ? -1 : ( A->b > B->b ) ? 1 :
          ( A->e < B->e ) ? -1 : ( A->e > B->e );
}

int fEndCmp( const void * a, const void * b )
{
   CEnd * A = (CEnd *) a, * B = (CEnd *) b;

   return ( A->x < B->x ) ? -1 : ( A->x > B->x ) ? 1 :
          ( A->cKind < B->cKind ) ? -1 : ( A->cKind > B->cKind );
}

void pUniteIntervals( void )
{
   int nStackSize = 0, i;

   for ( i = 0; i < nEndCnt; i++ )
   {
      if ( vEnds[i].cKind == 'b' )
      {
         if ( ++nStackSize == 1 )
            if ( nFinalIntervalCnt &&
                 ( vEnds[i].x - vFinalIntervals[ nFinalIntervalCnt - 1 ].e <= 0.01 ) )
               nFinalIntervalCnt--;
            else
               vFinalIntervals[ nFinalIntervalCnt ].b = vEnds[i].x;
      }
      else
         if ( --nStackSize == 0 )
            vFinalIntervals[ nFinalIntervalCnt++ ].e = vEnds[i].x;
   }
}

int main( void )
{
   int i;

   char sTemp[200];
   FILE * f = fopen( "light.c", "r" ); fgets( sTemp, 200, f );
//   FILE * f = fopen( "light.in", "r" );
   
   fscanf( f, "%d", &n ); fscanf( f, "%ld %ld", &b.x, &b.y );
   for ( i = 0; i < n; i++ )
   {
      CCircle c;
      fscanf( f, "%ld %ld %ld", &c.c.x, &c.c.y, &c.r );
      pAddCircle( &c );
   }
   
   fclose(f);

   qsort( vIntervals, nIntervalCnt, sizeof(CInterval), fIntervalCmp );

   for ( i = 0; i < nIntervalCnt; i++ )
   {
      vEnds[ nEndCnt ].x = vIntervals[i].b;
      vEnds[ nEndCnt ].cKind = 'b';
      vEnds[ nEndCnt++ ].nInterval = i;

      vEnds[ nEndCnt ].x = vIntervals[i].e;
      vEnds[ nEndCnt ].cKind = 'e';
      vEnds[ nEndCnt++ ].nInterval = i;
   }

   qsort( vEnds, nEndCnt, sizeof(CEnd), fEndCmp );
   
   pUniteIntervals();

   f = stdout;
// f = fopen( "light.out", "w" );
   
   if ( nFinalIntervalCnt )
   {
      fprintf( f, "%.2lf %.2lf", vFinalIntervals[0].b, vFinalIntervals[0].e );
      for ( i = 1; i < nFinalIntervalCnt; i++ )
         fprintf( f, "\n%.2lf %.2lf", vFinalIntervals[i].b, vFinalIntervals[i].e );
   }
   
   fclose(f);
   
   return 0;
}